@@ -34,7 +34,21 @@ sub parse_args {
     my($wantarray,$string) = @_;
     return unless defined $string and $string;
     if(defined $wantarray and $wantarray) {
-	return map { Apache::unescape_url_info($_) } split /[=&]/, $string;
+	my(@retval);
+	for (split /&/, $string) {
+	    my($k,$v); 
+	    if(/^(.+?)=(.*)$/) {
+		$k = $1; Apache->taint($k);
+		$v = $2; Apache->taint($v);
+		push @retval, 
+		unescape_url_info($k),
+		unescape_url_info($v);
+	    }
+	    else {
+		push @retval,
+		unescape_url_info($_), undef;
+	    }
+	}
     }
     $string;
 }
@@ -97,6 +111,12 @@ sub READLINE {
 
 *PRINT = \&print;
 
+sub PRINTF {
+    my $r = shift;
+    my $fmt = shift;
+    $r->print(sprintf($fmt, @_));
+}
+
 sub send_cgi_header {
     my($r, $headers) = @_;
     my $dlm = "\015?\012"; #a bit borrowed from LWP::UserAgent
